import processing.pdf.*;

// choose output
boolean pdf = true; //false = screen;
int volume = 0; // book volume
int pages = 512; // 512
String filename = "Kraftwerk-Computer_Love.mp3";

int perFrame = 512 * 8;
float factor = 0.72;
byte bytes[];
int w = 425; //433
int h = 687; //701
int margin = 50;
PFont font;
int hor = 128;
int ver = 128;
int book_size = hor * ver / 8 * pages;
int b = volume * book_size;
int book_end = b + book_size;
int mod = hor * ver / 8; // bits in a byte
float charWidth = (w - margin * 2.0) / (hor - 1);
float charHeight = (h - margin * 2.0) / (ver - 1);
float size = 1.618;
int page = volume * pages;
int pause = 5000;

void setup(){
  if (pdf){
    size(int(w * factor), int(h * factor), PDF, "book_" + volume + ".pdf");
    perFrame = 0;
  } else {
    factor = 1;
    size(int(w * factor), int(h * factor), P3D);
    smooth(4);
  }
  background(255);
  
  bytes = loadBytes(filename);
  font = createFont("Sys", 32);
  textFont(font);
  textAlign(CENTER, CENTER);
  textSize(14 * factor);
  // main page
  fill(0);
  text(filename + "\nVolume: " + (volume + 1) , w / 2.0 * factor, h / 2 * factor);
  noFill();
  stroke(0);
  strokeWeight(factor / 4);
  ellipseMode(RADIUS);
  textSize(7 * factor);
  
}

void draw(){
  if (millis() < pause) return;
  for(int i = 0; i < (perFrame == 0 ? bytes.length : perFrame); i++){
    if (b % mod == 0) {
      if (pdf){
        if (b == volume * book_size) nextPage();
        nextPage();
      } else {
        pushStyle();
        fill(255);
        noStroke();
        rect(0, 0, width, height);
        popStyle();
      }
      pushStyle();
      fill(0);
      noStroke();
      text(++page, (w / 2) * factor, (h - margin / 2) * factor);
      popStyle(); 
    }
    char[] text = binary(bytes[b]).toCharArray();
    float segment = b % 16;
    float y = b / 16 % hor * charHeight + margin;
    for(int j = 0; j < 8; j++){
      float xpos = (margin + (j + segment * 8) * charWidth) * factor;
      //text(text[j], xpos, y * factor); 
      switch(text[j]){
        case '0':
          ellipse(xpos, y * factor, size / 2 * factor, size * factor);
          break;
        case '1':
          line(xpos, (y - size) * factor, xpos, (y + size) * factor);
          break;
      }
    }
    // loop control
    if (++b >= book_end || b >= bytes.length) {
      println(" Done");
      if (pdf){
        exit();
        break;
      } else {
        noLoop();
        break;
      }
    }
  }
}


void nextPage(){
    PGraphicsPDF pdfg = (PGraphicsPDF) g;
    pdfg.nextPage();
    if (pdf) print('.');
}

